@@ -10,10 +10,11 @@ Date: April 5th, 2021.
 import json
 import logging
 import os
+from concurrent.futures import ThreadPoolExecutor
 from json import JSONDecodeError
 from urllib.parse import quote
 
-from oauthlib.oauth2 import BackendApplicationClient
+from oauthlib.oauth2 import BackendApplicationClient, OAuth2Token
 from requests_oauthlib import OAuth2Session
 
 from . import status
@@ -32,20 +33,8 @@ from .exceptions import (
     PermissionDenied,
     WebAPIUnavailable,
 )
-from .typing import (
-    Any,
-    Dict,
-    ExpandDict,
-    ExpandKeys,
-    ExpandValues,
-    FilterType,
-    List,
-    MethodType,
-    Optional,
-    OrderbyType,
-    Type,
-)
-from .utils import error_simplification_available, get_token, sentinel, set_token
+from .typing import Any, Dict, ExpandDict, ExpandKeys, ExpandValues, FilterType, List, MethodType, Optional, OrderbyType
+from .utils import cache, error_simplification_available, sentinel
 
 
 __all__ = ["DynamicsClient"]
@@ -58,10 +47,12 @@ class DynamicsClient:  # pylint: disable=R0904,R0902
     """Dynamics client for making queries from a Microsoft Dynamics 365 database."""
 
     request_counter: int = 0
-    actions_class: Type[Actions] = Actions
-    functions_class: Type[Functions] = Functions
+    cache_key: str = "dynamics-client-token"
     simplified_error_message: str = "There was a problem communicating with the server."
 
+    actions = Actions()
+    functions = Functions()
+
     error_dict = {
         status.HTTP_400_BAD_REQUEST: ParseError,
         status.HTTP_401_UNAUTHORIZED: AuthenticationFailed,
@@ -92,18 +83,13 @@ class DynamicsClient:  # pylint: disable=R0904,R0902
 
         self._api_url = api_url.rstrip("/") + "/"
         self._session = OAuth2Session(client=BackendApplicationClient(client_id=client_id))
-        token = get_token()
+        token = self.get_token()
         if token is None:  # pragma: no cover
             token = self._session.fetch_token(token_url=token_url, client_secret=client_secret, scope=scope)
-            set_token(token)
+            self.set_token(token)
         else:
             self._session.token = token
 
-        self.actions = self.actions_class(client=self)
-        """Predefined dynamics API actions. Used to make certain changes to data."""
-        self.functions = self.functions_class(client=self)
-        """Predefined dynamics API functions. Used to fetch certain types of data."""
-
         self._select: List[str] = []
         self._expand: ExpandDict = {}
         self._filter: FilterType = []
@@ -128,6 +114,27 @@ class DynamicsClient:  # pylint: disable=R0904,R0902
     def __setitem__(self, key, value):
         self.headers[key] = value
 
+    def get_token(self) -> OAuth2Token:
+        """Get dynamics client token in a thread, so it can be done in an async context."""
+
+        def task() -> OAuth2Token:
+            return cache.get(self.cache_key, None)
+
+        with ThreadPoolExecutor() as executor:
+            future = executor.submit(task)
+            return future.result()
+
+    def set_token(self, token: OAuth2Token):
+        """Set dynamics client token in a thread, so it can be done in an async context."""
+
+        def task():
+            expires = int(token["expires_in"]) - 60
+            cache.set(self.cache_key, token, expires)
+
+        with ThreadPoolExecutor() as executor:
+            future = executor.submit(task)
+            return future.result()
+
     @classmethod
     def from_environment(cls):
         """Create a client from environment variables:
@@ -550,6 +557,18 @@ class DynamicsClient:  # pylint: disable=R0904,R0902
         elif self.headers.get("Prefer") == 'odata.include-annotations="*"':
             self.headers.pop("Prefer")
 
+    @property
+    def suppress_duplicate_detection(self):
+        """If set to True, allow creating duplicate records if
+        Dynamics detects one during a POST or PATCH request.
+        https://docs.microsoft.com/en-us/powerapps/developer/data-platform/webapi/manage-duplicate-detection-create-update
+        """
+        return self.headers.get("MSCRM.SuppressDuplicateDetection") == "true"
+
+    @suppress_duplicate_detection.setter
+    def suppress_duplicate_detection(self, value: bool) -> None:
+        self.headers["MSCRM.SuppressDuplicateDetection"] = "true" if value else "false"
+
     @property
     def select(self) -> List[str]:
         """Get current $select statement."""
@@ -564,7 +583,7 @@ class DynamicsClient:  # pylint: disable=R0904,R0902
         if values is sentinel:
             values = self._select
 
-        return "$select=" + ",".join([key for key in values]) if values else ""
+        return "$select=" + ",".join(list(values)) if values else ""
 
     @property
     def expand(self) -> ExpandDict:
