@@ -5,9 +5,9 @@ Most common case is the separation of non-existing values vs. explicit `null` re
 """
 
 from datetime import datetime
-from typing import Any, Optional
 
-from dynamics.utils import from_dynamics_date_format
+from .typing import Any, Optional
+from .utils import from_dynamics_date_format
 
 
 __all__ = [
@@ -22,7 +22,7 @@ def as_int(value: Any, default: int = 0) -> int:
     try:
         if isinstance(value, str):
             value = value.replace(",", ".")
-        return int(value)
+        return int(float(value))
     except (ValueError, TypeError):
         return default
 
@@ -38,7 +38,7 @@ def as_float(value: Any, default: float = 0.0) -> float:
 
 def as_str(value: Any, default: str = "") -> str:
     try:
-        if value in (True, False, None):
+        if value is True or value is False or value is None:
             return default
         return str(value)
     except (ValueError, TypeError):
@@ -48,12 +48,12 @@ def as_str(value: Any, default: str = "") -> str:
 def as_bool(value: Any, default: bool = False) -> bool:
     try:
         return bool(value)
-    except (ValueError, TypeError):
+    except Exception:  # pylint: disable=W0703
         return default
 
 
 def str_as_datetime(value: str, default: Any = None) -> Optional[datetime]:
     try:
         return from_dynamics_date_format(value)
-    except Exception:
+    except Exception:  # pylint: disable=W0703
         return default
