@@ -1,14 +1,11 @@
 import logging
 import pickle
 import sqlite3
-from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime, timedelta, timezone
 from functools import wraps
 from pathlib import Path
 from uuid import UUID
 
-from oauthlib.oauth2 import OAuth2Token
-
 from .exceptions import DynamicsException
 
 
@@ -17,7 +14,7 @@ try:
 except ImportError:
     from backports.zoneinfo import ZoneInfo
 
-from .typing import TYPE_CHECKING, Any, List, Optional, Type
+from .typing import TYPE_CHECKING, Any, Callable, List, Optional, P, T, Type
 
 
 if TYPE_CHECKING:
@@ -31,8 +28,6 @@ __all__ = [
     "is_valid_uuid",
     "SQLiteCache",
     "cache",
-    "get_token",
-    "set_token",
     "error_simplification_available",
 ]
 
@@ -83,7 +78,7 @@ def from_dynamics_date_format(date: str, to_timezone: str = "UCT") -> datetime:
     return local_time
 
 
-def sqlite_method(method):
+def sqlite_method(method: Callable[P, T]) -> Callable[P, T]:
     """Wrapped method is executed under an open sqlite3 connection.
     Method's class should contain a 'self.connection_string' that is used to make the connection.
     This decorator then updates a 'self.con' object inside the class to the current connection.
@@ -92,7 +87,7 @@ def sqlite_method(method):
     """
 
     @wraps(method)
-    def inner(*args, **kwargs):
+    def inner(*args: P.args, **kwargs: P.kwargs) -> T:
         self = args[0]
         self.con = sqlite3.connect(self.connection_string)
         self._apply_pragma()  # pylint: disable=W0212
@@ -117,7 +112,7 @@ class SQLiteCache:
 
     DEFAULT_TIMEOUT = 300
     DEFAULT_PRAGMA = {
-        "mmap_size": 2 ** 26,  # https://www.sqlite.org/pragma.html#pragma_mmap_size
+        "mmap_size": 2**26,  # https://www.sqlite.org/pragma.html#pragma_mmap_size
         "cache_size": 8192,  # https://www.sqlite.org/pragma.html#pragma_cache_size
         "wal_autocheckpoint": 1000,  # https://www.sqlite.org/pragma.html#pragma_wal_autocheckpoint
         "auto_vacuum": "none",  # https://www.sqlite.org/pragma.html#pragma_auto_vacuum
@@ -200,31 +195,7 @@ except ImportError:
     cache = SQLiteCache()
 
 
-def get_token() -> OAuth2Token:
-    """Get dynamics client token in a thread, so it can be done in an async context."""
-
-    def task() -> OAuth2Token:
-        return cache.get("dynamics-client-token", None)
-
-    with ThreadPoolExecutor() as executor:
-        future = executor.submit(task)
-        return future.result()
-
-
-def set_token(token: OAuth2Token):
-    """Set dynamics client token in a thread, so it can be done in an async context."""
-
-    def task():
-        name = "dynamics-client-token"
-        expires = int(token["expires_in"]) - 60
-        cache.set(name, token, expires)
-
-    with ThreadPoolExecutor() as executor:
-        future = executor.submit(task)
-        return future.result()
-
-
-def error_simplification_available(func):
+def error_simplification_available(func: Callable[P, T]) -> Callable[P, T]:
     """Errors in the function decorated with this decorator can be simplified to just a
     DynamicsException with default error message using the keyword: 'simplify_errors'.
     This is useful if you want to hide error details from frontend users.
@@ -236,7 +207,7 @@ def error_simplification_available(func):
     """
 
     @wraps(func)
-    def inner(*args, **kwargs):
+    def inner(*args: P.args, **kwargs: P.kwargs) -> T:
         simplify_errors: bool = kwargs.pop("simplify_errors", False)
         raise_separately: List[Type[Exception]] = kwargs.pop("raise_separately", [])
 
